/**
 * @file conf.c  Configuration utils
 *
 * Copyright (C) 2010 Creytiv.com
 */
#define _BSD_SOURCE 1
#include <fcntl.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <stdio.h>
#include <sys/stat.h>
#ifdef HAVE_IO_H
#include <io.h>
#endif
#include <re.h>
#include <rem.h>
#include <baresip.h>
#include "core.h"


#define DEBUG_MODULE "conf"
#define DEBUG_LEVEL 5
#include <re_dbg.h>


#if defined (WIN32) || defined (__SYMBIAN32__)
#define DIR_SEP "\\"
#else
#define DIR_SEP "/"
#endif


static struct conf *conf_obj;



static void print_populated(const char *what, uint32_t n)
{
	(void)re_printf("Populated %u %s%s\n", n, what, 1==n ? "" : "s");
}


/**
 * Load all modules from config file
 *
 * @return 0 if success, otherwise errorcode
 */
int conf_modules(void)
{
#if 0
	char path[256], file[256];
	int err;

	err = conf_path_get(path, sizeof(path));
	if (err)
		return err;

	if (re_snprintf(file, sizeof(file), "%s/config", path) < 0)
		return ENOMEM;

	err = conf_alloc(&conf_obj, file);
	if (err)
		goto out;

	err = module_init(conf_obj);
	if (err) {
		DEBUG_WARNING("configure module parse error (%m)\n", err);
		goto out;
	}

	print_populated("audio codec",  list_count(aucodec_list()));
	print_populated("audio filter", list_count(aufilt_list()));
#ifdef USE_VIDEO
	print_populated("video codec",  list_count(vidcodec_list()));
	print_populated("video filter", list_count(vidfilt_list()));
#endif

 out:
	conf_obj = mem_deref(conf_obj);
	return err;
#else
	return 0;
#endif
}


/**
 * Get the current configuration object
 *
 * @return Config object
 *
 * @note It is only available during init
 */
struct conf *conf_cur(void)
{
	return conf_obj;
}

int configure(void)
{
	struct pl pollm;
	enum poll_method method;
	uint32_t v;
	int err = 0;

	struct config * cfg = conf_config();	

#if 0
	/* Core */
	if (0 == conf_get(conf, "poll_method", &pollm)) {
		if (0 == poll_method_type(&method, &pollm)) {
			err = poll_method_set(method);
			if (err) {
				DEBUG_WARNING("poll method (%r) set: %s\n",
						  &pollm, strerror(err));
			}
		}
		else {
			DEBUG_WARNING("unknown poll method (%r)\n", &pollm);
		}
	}
#endif
	struct pl dummy;
	struct pl modname;
	pl_set_str(&dummy, "");

	//dns_server_handler(&dummy, NULL);

	if (cfg->codecs.g711)
	{
		pl_set_str(&modname, "g711");
		load_module2(NULL, &modname);
	}

	if (cfg->codecs.g726_32)
	{
		pl_set_str(&modname, "g726_32");
		load_module2(NULL, &modname);
	}

	if (cfg->codecs.gsm)
	{
		pl_set_str(&modname, "gsm");
		load_module2(NULL, &modname);
	}

	if (cfg->codecs.speex)
	{
		pl_set_str(&modname, "speex");
		load_module2(NULL, &modname);
	}

	if (cfg->sound_backend == SOUND_WINWAVE) {
		pl_set_str(&modname, "winwave");
	} else {
		// default: portaudio/DirectSound
		pl_set_str(&modname, "portaudio");
	}
	load_module2(NULL, &modname);

	//TODO: AEC seems to not work well
#if 0
	pl_set_str(&modname, "speex_aec");
	load_module2(NULL, &modname);
#endif

	pl_set_str(&modname, "stun");
	load_module2(NULL, &modname);


	return err;
}
